<?php
    function IsSecure() {
        return ( ! empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off') || $_SERVER['SERVER_PORT'] == 443;
    }
    
    function FindEPByBinding($list, $binding) {
        for ($i = 0; $i < $list->Count; $i++) {
            $ep = $list->get_Items($i)->toObject()->CastTo();
            if ($ep->get_Binding() == $binding)
                return $ep;
        }
        return NULL;
    }
    
    function ProcessRequest($doc, $login, $state, $spMetaList) {     
        $elem = $doc->get_DocumentElement();
        $localName = $elem->get_LocalName();
        
        if (strcmp($localName, 'AuthnRequest') !== 0) {
            header('HTTP/1.1 400 Bad Request', true, 400);
            exit;
        }
        
        $samlReq = new TElSAMLAuthnRequestElement();
        $samlReq->LoadFromXML($doc->get_DocumentElement());
        
        // find SP by issuer
        $count = count($spMetaList);
        
        // load metadata
        for ($i = 0; $i < $count; $i++) {
            $spInfo = new TElSAMLServiceProviderInfo();
            $spInfo->LoadMetadata($spMetaList[$i]);
            $spList[$i] = $spInfo;
        }
        
        for ($i = 0; $i < $count; $i++) {
            $id = $spList[$i]->get_EntityID();
            if (strcmp($id, $samlReq->get_Issuer()->get_Value()) == 0) {
                $currentSP = $spList[$i];
                break;
            }
        }
        
        if ( ! isset($currentSP)) {
            for ($i = 0; $i < $count; $i++) {
                $id = $spList[$i]->get_SPNameQualifier();
                if (strcmp($id, $samlReq->get_Issuer()->get_Value()) == 0) {
                    $currentSP = $spList[$i];
                    break;
                }
            }
        }
        
        $acsUrl = $samlReq->get_AssertionConsumerServiceURL();
        if ( ! isset($currentSP)) {
            for ($i = 0; $i < $count; $i++) {
                $spInfo = $spList[$i];
                
                for ($j = 0; $j < $spInfo->get_AssertionConsumerServiceCount(); $j++) {
                    $location = $spInfo->get_AssertionConsumerServices($j)->get_location();
                    if (strcmp($location, $acsUrl) == 0) {
                        $currentSP = $spList[$i];
                        break;
                    }
                }
            }
        }
        
        // SP not found
        if ( ! isset($currentSP)) {
            header('HTTP/1.1 500 Internal Error', true, 500);
            exit;
        }
        
        // reload request with signature checking
        $sigHandler = new TElSAMLSignatureHandler();
        $sigHandler->set_Certificate($currentSP->get_SigningCertificate());
        $sigHandler->set_ValidateReferences(true);
        
        $samlReq = new TElSAMLAuthnRequestElement();
        $samlReq->set_Handler($sigHandler);
        $samlReq->LoadFromXML($doc->get_DocumentElement());
        
        // check request
        if (strcmp($samlReq->get_Version(), SBXMLSAMLCore\SAMLVersion) !== 0) {
            header('HTTP/1.1 400 Bad Request', true, 400);
            exit;
        }
        
        $diff = SBUtils\DateTimeToUnixTime($samlReq->get_IssueInstant()) - SBUtils\DateTimeToUnixTime(SBUtils\UTCNow());
        if ($diff > MAX_ISSUE_INSTANT_TIME_DIFF) {
            header('HTTP/1.1 400 Bad Request', true, 400);
            exit;
        }
        
        $acsUrl = $samlReq->get_AssertionConsumerServiceURL();
        $acsIdx = $samlReq->get_AssertionConsumerServiceIndex();
        
        if ($acsIdx >= 0) {
            $ep = $currentSP->FindAssertionConsumerServiceEP($acsIdx);
            if (isset($ep)) {
                $acsUrl = $ep->get_Location();
                $bindingId = $ep->get_Binding();
            } else {
                header('HTTP/1.1 400 Bad Request', true, 400);
                exit;
            }
        } else if (strlen($acsUrl) > 0) {
            $epList = $currentSP->FindAssertionConsumerServiceEP($acsUrl);

            
            if ($epList->Count == 1) {
                $ep = $epList->get_Item(0);
                
                $acsUrl = $ep->get_Location();
                $bindingId = $ep->get_Binding();
                
                $bindingStr = $samlReq->get_ProtocolBinding();
                if (strlen($bindingStr) > 0) {
                    $bindingId = TElSAMLBinding\StrToBinding($bindingStr);
                }
            } else if ($epList->Count > 1) {
                $bindingStr = $samlReq->get_ProtocolBinding();
                
                if (strlen($bindingStr) > 0) {
                    $ep = FindEPByBinding($epList, TElSAMLBinding::StrToBinding($bindingStr));
                }
                
                if (isset($ep)) {
                    $acsUrl = $ep->get_Location();
                    $bindingId = $ep->get_Binding();
                } else {
                    header('HTTP/1.1 400 Bad Request', true, 400);
                    exit;
                }
            } else {
                header('HTTP/1.1 400 Bad Request', true, 400);
                exit;
            }
        } else {
            $ep = $currentSP->FindDefaultAssertionConsumerServiceEP();
            
            if ( ! isset($ep)) {
                $ep = $currentSP->FindAssertionConsumerServiceEP(TElSAMLBinding\StrToBinding(PREFFERED_SSO_RESP_BINDING));
                
                if (( ! isset($ep)) && ($currentSP->get_AssertionConsumerServiceCount() > 0)) {
                    $ep = $currentSP->get_AssertionConsumerServices(0);
                } else {
                    header('HTTP/1.1 500 Internal Error', true, 500);
                    exit;
                }
                
                $acsUrl = $ep->get_Location();
                $bindingId = $ep->get_Binding();
            }
        }
        
        // create response
        $samlResp = new TElSAMLResponseElement();
        
        $samlResp->set_Version(SBXMLSAMLCore\SAMLVersion);
        $samlResp->set_ID(SBRandom\SBRndString(20));
        $samlResp->set_IssueInstant(SBUtils\UTCNow());
        $samlResp->set_Destination($acsUrl);
        $samlResp->set_InResponseTo($samlReq->get_ID());
        
        $issuer = new TElSAMLIssuerElement();
        $issuer->set_Value(IDP_ISSUER);
        $issuer->set_Format("");
        $samlResp->set_Issuer($issuer);
        
        $statusCode = new TElSAMLStatusCodeElement();
        $statusCode->set_Value(SBXMLSAMLProtocol\xmlSAMLStatusSuccess);
        
        $status = new TElSAMLStatusElement();
        $status->set_StatusCode($statusCode);
        $samlResp->set_Status($status);
        
        // generate assertions
        $notBefore = SBUtils\DateTimeAddMillis(SBUtils\UTCNow(), NOT_BEFORE_TIMEOUT);
        $notOnOrAfter = SBUtils\DateTimeAddMillis($notBefore, ASSERTIONS_TTL);
        
        $assertion = new TElSAMLAssertionElement();
        $assertion->set_Version(SBXMLSAMLCore\SAMLVersion);
        $assertion->set_ID(SBRandom\SBRndString(20));
        $assertion->set_IssueInstant(SBUtils\UTCNow());
        
        $issuer = new TElSAMLIssuerElement();
        $issuer->set_Value(IDP_ISSUER);
        $issuer->set_Format("");
        $assertion->set_Issuer($issuer);
        
        $subject = new TElSAMLSubjectElement();
        $assertion->set_Subject($subject);
        
        $nameId = new TElSAMLNameIDElement();
        $subject->set_ID($nameId);
        $nameId->set_Format(SBXMLSAMLCore\xmlSAMLNameIDFormatTransient);
        $nameId->set_Value($login);
        
        $idx = $assertion->get_Subject()->AddSubjectConfirmation();
        $confirmation = $assertion->get_Subject()->get_SubjectConfirmations($idx);
        $confirmation->set__Method(SBXMLSAMLProtocol\xmlSAMLConfirmationMethodBearer);
        
        $confirmData = new TElSAMLSubjectConfirmationDataElement();
        $confirmation->set_SubjectConfirmationData($confirmData);
        $confirmData->set_Recipient($acsUrl);
        $confirmData->set_NotBefore($notBefore);
        $confirmData->set_NotOnOrAfter($notOnOrAfter);
        $confirmData->set_InResponseTo($samlReq->get_ID());
        $confirmData->set_AType("");
        $confirmData->set_Address("");
        
        $conditions = new TElSAMLConditionsElement();
        $assertion->set_Conditions($conditions);
        $conditions->set_NotBefore($notBefore);
        $conditions->set_NotOnOrAfter($notOnOrAfter);
        
        $entityID = $currentSP->get_EntityID();
        if (strlen($entityID) > 0) {
            $audience = new TElSAMLAudienceRestrictionElement();
            $idx = $audience->AddAudience();
            $audience->get_AudienceList($idx)->set_Value($entityID);
            $assertion->get_Conditions()->AddCondition($audience);
        }
        
        $authn = new TElSAMLAuthnStatementElement();
        $assertion->AddStatement($authn);

        $authn->set_AuthnInstant(SBUtils\UTCNow());
        $authn->set_SessionNotOnOrAfter(SBUtils\DateTimeAddMillis(SBUtils\UTCNow(), SAML_SESSION_TTL));
        $authn->set_SessionIndex(SBRandom\SBRndString(20));

        $authnCtx = new TElSAMLAuthnContextElement();
        $authn->set_AuthnContext($authnCtx);
        $authnCtx->set_Choice(TSBSAMLAuthnContextChoice::sacV1);
        $authnCtx->set_AuthnContextClassRef("");
        $authnCtx->set_AuthnContextDeclRef("");

        if (IsSecure())
            $authnCtx->set_AuthnContextClassRef(SBXMLSAMLProtocol\xmlSAMLAuthnContextClassPasswordProtectedTransport);
        else
            $authnCtx->set_AuthnContextClassRef(SBXMLSAMLProtocol\xmlSAMLAuthnContextClassPassword);

        $samlResp->AddAssertion($assertion);
        
        // sign response
        if (SIGN_RESPONSES) {
            $signCert = new TElX509Certificate(NULL);
            $signCert->LoadFromFileAuto(SIGN_CERT, "");
            
            $sigHandler = new TElSAMLSignatureHandler();
            $sigHandler->set_Certificate($signCert);
            
            $samlResp->set_Signed(true);
            $samlResp->set_Handler($sigHandler);
        }
        
        // wrap response
        $doc = new TElXMLDOMDocument();
        $doc->AppendChild($samlResp->SaveToXML($doc));
        
        if ($bindingId == TSBSAMLBindingType::sbtRedirect) {
            $binding = new TElSAMLRedirectBinding();
            
            $binding->set_URL($acsUrl);
            $binding->set_RelayState($state);
            $binding->set_SignatureAlg(SBXMLDefs\xmlSignatureAlgorithmRSASHA1);
            
            if ($samlResp->get_Signed())
                $binding->set_KeyMaterial($samlResp->get_Handler()->get_Certificate()->get_KeyMaterial());
            
            $respStr = $binding->SaveResponse($doc);
        } else if ($bindingId == TSBSAMLBindingType::sbtPOST) {
            $binding = new TElSAMLPOSTBinding();
            
            $binding->set_URL($acsUrl);
            $binding->set_RelayState($state);
            $binding->set_Mode(TSBSAMLPOSTBindingMode::spmServer);
            
            $respStr = $binding->SaveResponse($doc);
        }
        
        // return response
        if ($bindingId == TSBSAMLBindingType::sbtRedirect) {
            header('HTTP/1.1 302 Moved Temporarily', true, 302);
            header('Location: ' . $respStr);
        } else if ($bindingId == TSBSAMLBindingType::sbtPOST) {
            echo $respStr;
        } else {
            header('HTTP/1.1 500 Internal Error', true, 500);
            exit;
        }
        
        $spList = null;
        $currentSP = null;
    }
?>
